# -*- coding: utf-8 -*-

"""Main module."""
import json
import requests
from typing import *
from .lib import util
from pyrsistent import freeze, pvector
from . import helper
import os

# fr.deploy(name='EchoHandler',
#               handler=e,
#               url='https://www.15rock.com',
#               credentials={'u': 'r', 'p': 'pp'},
#               project='p1'
#               )
REQUIREMENTS_FILE = 'requirements.txt'


class BaseHandler(object):
    pass


def deploy(project: str, function: str, url: str = None, main_file: str = None, dependencies: List = None,
           requirements_file: str = None, runtime: str = None, runtime_version: str = None, project_folder: str = None,
           credentials: Dict[str, str] = None, credentials_file: str = None, spec: Dict = None, metadata: Dict = None,
           config_file: str = None):
    """

    :param main_file:
    :param credentials:
    :param credentials_file: Path to the credentials file
    :param url:
    :param project:
    :param dependencies: files or folders not in your project_folder which you want to add to your deployment package.
        An example could be an external secrets file or some shared code outside your repo(which is not a pip
        library for e.g.)
    :param function:
    :param requirements_file:
    :param runtime:
    :param runtime_version:
    :param project_folder:
    :param spec: Dict of resources, image setup commands etc.
    :param metadata: Dict of labels, annotations etc.
    :param config_file: Config file. Any values passed in explicitly will override any values in the config file.
    Note, this pertains only to `spec` and `metadata` keys right now. i.e. if spec is provided, it will __completely__
    replace any values specified in the *config_file*
    :return:
    """
    tmp_folder = util.tmp_folder()

    credentials = helper.get_credentials(credentials, credentials_file)
    config = helper.read_config(config_file)
    url = url or 'https://app.15rock.com/gateway/compute'
    main_file = main_file or helper.default_file_path('main.py')

    util.fail_if_no_file(main_file)
    requirements_file = requirements_file or helper.default_file_path(REQUIREMENTS_FILE) or str(
        util.make_requirements_file(tmp_folder))
    # project_folder = project_folder or os.getcwd()

    # requirements_file = requirements_file or str(util.make_requirements_file(tmp_folder))
    # name = name or util.file_name(main_file)
    dependencies = make_dependencies(dependencies, main_file, project_folder, requirements_file)

    spec = spec or config.get('spec') or dict()
    metadata = metadata or config.get('metadata') or dict()
    runtime = runtime or 'python'
    runtime_version = runtime_version or '3.6'

    runtime_path = tmp_folder / 'runtime.txt'
    runtime_path.write_text(runtime + '-' + runtime_version)
    credentials_tmp_path = tmp_folder / helper.FR_FILE_NAME
    dependencies = dependencies.append(str(runtime_path))

    util.write_credentials(credentials_tmp_path, credentials)
    dependencies = dependencies.append(str(credentials_tmp_path))
    # project_folder = project_folder or util.determine_project_folder()

    # dependencies = dependencies.extend([fifteenrock])

    # dependency_paths = [util.module_path(d) for d in dependencies]

    dependency_paths = pvector(util.module_path(d) for d in dependencies)

    optional = util.optional(project=project, name=function, spec=spec, metadata=metadata)
    mandatory = dict(credentials=credentials, main_file_name=os.path.basename(main_file))
    raw_data = {**mandatory, **optional}
    compute_url = url + '/api/v0'
    new_url = compute_url + '/deploy'

    archive_file = tmp_folder / 'archive.zip'

    util.archive(archive_file, dependency_paths)

    headers = dict(apikey=credentials['faas_token'])
    try:
        deploy_result = deploy_to_compute(archive_file, headers, new_url, raw_data)
        poll_result = poll_if_function_created(deploy_result['function_id'], compute_url, headers)
        return dict(message=[f"Function available at {poll_result['function_url']}"])
    except Exception as e:
        raise e
    finally:
        util.remove_dir(tmp_folder)


def make_dependencies(dependencies, main_file, project_folder, requirements_file):
    dependencies = dependencies or []
    dependencies = freeze(dependencies)
    dependencies = dependencies.append(main_file)
    dependencies = dependencies.append(requirements_file)
    if project_folder:
        dependencies = dependencies.append(project_folder)
    return dependencies


# def poll_if_function_created(function_id, compute_url: str, headers: Dict):
#     tries = range(60)
#     interval = 30
#
#     import time
#
#     raw_data = dict(function_id=function_id)
#     status_url = compute_url + '/function_status'
#     new_headers = {**headers, **{"Content-type": "application/json"}}
#     for i in tries:
#         print(f'Try:{i}')
#         result = util.get_json(status_url, raw_data=raw_data, headers=new_headers)
#         code = result['code']
#         if code == 200:
#             return result
#         elif code == 40:
#             raise ValueError('Error in creating function')
#         # if code == 400:
#         #     # TODO: Report actual error
#         #     raise ValueError('Error in creating function')
#         # elif code == 200:
#         #     return result
#         time.sleep(interval)
#
#     # Else raise Timeout error
#     raise ValueError('Function creation timed out. Please try again.')

def poll_if_function_created(function_id, compute_url: str, headers: Dict):
    tries = range(60)
    interval = 30

    import time

    raw_data = dict(function_id=function_id)
    status_url = compute_url + '/function_status'
    new_headers = {**headers, **{"Content-type": "application/json"}}
    for i in tries:
        print(f'Try:{i}')
        response = util.get_json(status_url, raw_data=raw_data, headers=new_headers)
        if response:
            result = response.json()
            code = result['code']
            if code == 200:
                return result
        else:
            raise ValueError('Error in creating function')

        time.sleep(interval)

    # Else raise Timeout error
    raise ValueError('Function creation timed out. Please try again.')


def deploy_to_compute(archive_file, headers, new_url, raw_data):
    return util.post(new_url, raw_data=raw_data, files={'archive': archive_file}, headers=headers)


# class DatabaseClient(object):
#     def __init__(self, url: str, credentials: Dict):
#         # self._url = url
#         self._url = url + '/' + 'v0'
#         self.credentials = credentials
#         pass
#
#     def url(self):
#         return self._url
#
#     # def drop(self, schema, table):
#     #     drop_query = f"DROP TABLE IF EXISTS {schema}.{table};"
#     #
#     #     self.execute_command(command=drop_query)
#
#     def query_sql(self, query: str):
#         raw_data = dict(credentials=self.credentials, query=query)
#         return self.post_json('/query_sql', raw_data)
#
#         # return result.json()
#
#     def post_json(self, url_part: str, raw_data: Dict):
#
#         headers = {"Content-type": "application/json"}
#
#         a_url = self.url() + url_part
#         try:
#             result = requests.post(a_url, headers=headers, data=json.dumps(raw_data))
#             js = result.json()
#             result.raise_for_status()
#             return js
#
#         except requests.exceptions.RequestException as e:
#             print(f"ROCK_ERROR:{json.dumps(js)}")
#             raise e
#
#     def execute_command(self, command: str):
#         result = self.post_json('/execute_command', dict(credentials=self.credentials, query=command))
#         # TODO: Start of GRAPHQL Code. Commenting for now
#         # tables = get_tables_to_track(query)
#         #
#         # for t in tables:
#         #
#         #     schema = t[0]
#         #     self.query_hasura([track_table(schema, t[1])])
#         #     table_info = self.query_hasura_single(get_table_info_for_relationships(schema))
#         #     if table_info:
#         #         fkc_dicts = get_foreign_key_constraints(table_info)
#         #         for fkc in fkc_dicts:
#         #             rel = get_object_relationship(fkc)
#         #             obj_rel_cmd = create_object_relationship(rel['label'], rel['schema'], rel['table_name'],
#         #                                                      rel['foreign_key'])
#         #             self.query_hasura_single(obj_rel_cmd)
#         #         for fkc in fkc_dicts:
#         #             rel = get_array_relationship(fkc)
#         #             arr_rel_cmd = create_array_relationship(rel['label'], rel['ref_schema'], rel['ref_table'],
#         #                                                     rel['schema'], rel['table_name'], rel['foreign_key'])
#         #             self.query_hasura_single(arr_rel_cmd)
#         # TODO: End of GRAPHQL Code. Commenting for now
#         return result
#
#     @staticmethod
#     def insert_query(table_name: str, columns: List, values: List):
#         cn = ', '.join(columns)
#         vn = []
#         for v in values:
#             if type(v) != str:
#                 vx = str(v)
#             else:
#                 vx = "'" + v + "'"
#             vn.append(vx)
#         vn = ', '.join(vn)
#         return f"INSERT INTO {table_name} ({cn}) VALUES ({vn});"
#
#     def insert_sql(self, table, columns, values):
#         inserts = [self.insert_query(table, columns, vn) for vn in values]
#         self.execute_command(command=''.join(inserts))
#
#     # def read(self, columns: List[str]):
#     #
#     #     return self.post_json('/read', dict(credentials=self.credentials, columns=json.dumps(columns)))
#
#     # def query_graphl(self, query: str):
#     #     request = requests.post(self.url() + "/graphql", json={'query': query})
#     #     if request.status_code == 200:
#     #         result = request.json()
#     #     else:
#     #         raise Exception("Query failed to run by returning code of {}. {}".format(request.status_code, query))
#     #
#     #     return result
#
#     # def query_graphl(self, query: str):
#     #     return self.post_json('/graphql', dict(query=query))
#     #
#     # def query_hasura(self, query_dict: List[Dict]):
#     #     result = []
#     #     for q in query_dict:
#     #         r = self.post_json('/hasura_api', dict(query=q))
#     #         result.append(r)
#     #     return result
#     #
#     # def query_hasura_single(self, query_dict: Dict):
#     #     return self.post_json('/hasura_api', dict(query=query_dict))
#

class DatabaseClient(object):
    def __init__(self, url: str, credentials: Dict):
        # self._url = url
        # self._url = url + '/' + 'v0'
        self._url = url + "/api/v0/db"
        self.credentials = credentials
        pass

    def url(self):
        return self._url

    # def drop(self, schema, table):
    #     drop_query = f"DROP TABLE IF EXISTS {schema}.{table};"
    #
    #     self.execute_command(command=drop_query)

    def query_sql(self, query: str):
        raw_data = dict(credentials=self.credentials, query=query)
        return self.post_json('/query_sql', raw_data)

        # return result.json()

    def post_json(self, url_part: str, raw_data: Dict):
        headers = {"Content-type": "application/json",
                   "apikey": self.credentials['faas_token']}

        a_url = self.url() + url_part
        try:
            result = requests.post(a_url, headers=headers, data=json.dumps(raw_data), verify=False)
            js = result.json()
            result.raise_for_status()
            return js

        except requests.exceptions.RequestException as e:
            print(f"ROCK_ERROR:{json.dumps(js)}")
            raise e

    def execute_command(self, command: str):
        result = self.post_json('/execute_command', dict(credentials=self.credentials, query=command))
        # TODO: Start of GRAPHQL Code. Commenting for now
        # tables = get_tables_to_track(query)
        #
        # for t in tables:
        #
        #     schema = t[0]
        #     self.query_hasura([track_table(schema, t[1])])
        #     table_info = self.query_hasura_single(get_table_info_for_relationships(schema))
        #     if table_info:
        #         fkc_dicts = get_foreign_key_constraints(table_info)
        #         for fkc in fkc_dicts:
        #             rel = get_object_relationship(fkc)
        #             obj_rel_cmd = create_object_relationship(rel['label'], rel['schema'], rel['table_name'],
        #                                                      rel['foreign_key'])
        #             self.query_hasura_single(obj_rel_cmd)
        #         for fkc in fkc_dicts:
        #             rel = get_array_relationship(fkc)
        #             arr_rel_cmd = create_array_relationship(rel['label'], rel['ref_schema'], rel['ref_table'],
        #                                                     rel['schema'], rel['table_name'], rel['foreign_key'])
        #             self.query_hasura_single(arr_rel_cmd)
        # TODO: End of GRAPHQL Code. Commenting for now
        return result

    @staticmethod
    def insert_query(table_name: str, columns: List, values: List):
        cn = ', '.join(columns)
        vn = []
        for v in values:
            if type(v) != str:
                vx = str(v)
            else:
                vx = "'" + v + "'"
            vn.append(vx)
        vn = ', '.join(vn)
        return f"INSERT INTO {table_name} ({cn}) VALUES ({vn});"

    def insert_sql(self, table, columns, values):
        inserts = [self.insert_query(table, columns, vn) for vn in values]
        self.execute_command(command=''.join(inserts))

    # def read(self, columns: List[str]):
    #
    #     return self.post_json('/read', dict(credentials=self.credentials, columns=json.dumps(columns)))

    # def query_graphl(self, query: str):
    #     request = requests.post(self.url() + "/graphql", json={'query': query})
    #     if request.status_code == 200:
    #         result = request.json()
    #     else:
    #         raise Exception("Query failed to run by returning code of {}. {}".format(request.status_code, query))
    #
    #     return result

    # def query_graphl(self, query: str):
    #     return self.post_json('/graphql', dict(query=query))
    #
    # def query_hasura(self, query_dict: List[Dict]):
    #     result = []
    #     for q in query_dict:
    #         r = self.post_json('/hasura_api', dict(query=q))
    #         result.append(r)
    #     return result
    #
    # def query_hasura_single(self, query_dict: Dict):
    #     return self.post_json('/hasura_api', dict(query=query_dict))


def get_database_client(url: str = "https://app.15rock.com/gateway/compute",
                        credentials: Dict = None,
                        credentials_path: str = None) -> DatabaseClient:
    credentials = credentials or helper.get_credentials(credentials, credentials_path)
    return DatabaseClient(url, credentials)


# def is_create_cmd(stmt):
#     return stmt.get_type() == 'CREATE'
#     pass


# def flatten(ll):
#     return [lu for l in ll for lu in l]

#
# def is_create_table_stmt(s):
#     # return s.get_type() == 'CREATE' and s.parent.get_type()
#     return 'create table' in s.value.lower()


# def get_tables_to_track(ddl):
#     parsed = sqlparse.parse(ddl)
#     create_stmts = list(filter(is_create_table_stmt, parsed))
#     all_tokens = flatten(map(lambda s: s.tokens, create_stmts))
#
#     table_tokens = list(filter(lambda t: t._get_repr_name() == 'Identifier', all_tokens))
#     #
#     return list(map(lambda tt: (tt.get_parent_name(), tt.get_name()), table_tokens))
#     pass


# def get_query(columns: List[str], meta, engine):
#     splits = [c.split('.') for c in columns]
#
#     full_columns = []
#     for s in splits:
#         if len(s) == 2:
#             schema_name = "public"
#             table_name = s[0]
#             column_name = s[1]
#         else:
#             schema_name = s[0]
#             table_name = s[1]
#             column_name = s[2]
#         full_columns.append(FullColumn(schema_name, table_name, column_name))
#
#     full_tables = []
#     for s in splits:
#         if len(s) == 2:
#             schema_name = "public"
#             table_name = s[0]
#         else:
#             schema_name = s[0]
#             table_name = s[1]
#         full_tables.append(FullTable(schema_name, table_name))
#
#     new_tables = inner_join_strs(set(full_tables), meta, engine)
#     fc_strs = ['.'.join([fc.schema_name, fc.table_name, fc.column_name]) for fc in full_columns]
#     new_columns = ', '.join(fc_strs)
#
#     query = f'''SELECT {new_columns} from {new_tables};'''
#
#     return query
#
#
# def get(user: str, columns: List[str]):
#     meta = MetaData()
#     q = get_query(columns, meta, db)
#     return read_sql(user, q)
#     pass


#
#
# def select_str(column_strs):
#     column_str = ', '.join(column_strs)
#     return f'''
# SELECT {column_str}
# '''
#
#
# def inner_join_str(pk_schema: str, pk_table: str, pk: str, fk_schema: str, fk_table: str, fk: str):
#     pk_table_full = f"{pk_schema}.{pk_table}"
#     fk_table_full = f"{fk_schema}.{fk_table}"
#     return f'''
# {pk_table_full} INNER JOIN {fk_table_full}
# ON {pk_table_full}.{pk} = {fk_table_full}.{fk}
# '''
#
#
# def to_alchemy_tables(tables: List[FullTable], meta, engine):
#     return [Table(t.table_name, meta, autoload=True, autoload_with=engine, schema=t.schema_name) for t in tables]
#
#
# def get_foreign_keys(alchemy_table):
#     return list(alchemy_table.foreign_keys)
#
#
# # pprint(inner_join_str(fk.column.table.schema, fk.column.table.name, fk.column.name, 'public', 'scoring_model',
# #                       fk.parent.name))
#
#
# def get_primary_full_table(fk):
#     return FullTable(fk.column.table.schema, fk.column.table.name)
#
#
# def exists(a_full_table, full_tables) -> bool:
#     for ft in full_tables:
#         if ft.schema_name == a_full_table.schema_name and ft.table_name == a_full_table.table_name:
#             return True
#     return False
#
#
# def inner_join_strs(full_tables, meta, engine):
#     final_joins = []
#     alchemy_tables = to_alchemy_tables(full_tables, meta, engine)
#     if len(full_tables) > 1:
#         for table in alchemy_tables:
#             fks = get_foreign_keys(table)
#             for fk in fks:
#                 pk_table = get_primary_full_table(fk)
#                 if exists(pk_table, full_tables):
#                     an_inner_join = inner_join_str(pk_table.schema_name, pk_table.table_name, fk.column.name,
#                                                    table.schema,
#                                                    table.name, fk.parent.name)
#                     final_joins.append(an_inner_join)
#         return final_joins[0]
#     else:
#         full_table = list(full_tables)[0]
#         return full_table.schema_name + '.' + full_table.table_name
#
#
# def query_graphl(query: str):
#     request = requests.post("http://localhost:8080/v1alpha1/graphql", json={'query': query})
#     if request.status_code == 200:
#         result = request.json()
#     else:
#         raise Exception("Query failed to run by returning code of {}. {}".format(request.status_code, query))
#
#     return result
#
#

def main(context, event):
    print('Hi')


if __name__ == '__main__':
    #     db_string = "postgres://postgres:@localhost:5432/postgres"
    #     # engine = create_engine(db_string)
    #     # meta = MetaData()
    #     assert get_query(columns=['public.customer.name']) == 'SELECT public.customer.name from public.customer;'
    #     assert get_query(columns=['public.customer.name',
    #                               'public.customer.age']) == 'SELECT public.customer.name, public.customer.age from public.customer;'
    #
    #     first_join_query = """SELECT public.customer.name, public.scoring_model.score from
    # public.customer INNER JOIN public.scoring_model
    # ON public.customer.id = public.scoring_model.customer_id
    # ;"""
    #     assert get_query(columns=["public.customer.name", "public.scoring_model.score"]) == first_join_query
    #     assert get_query(columns=["customer.name", "scoring_model.score"]) == first_join_query
    #     tables = [FullTable("public", "customer"),
    #               FullTable("public", "scoring_model")]
    #     final_join = inner_join_strs(tables)
    #
    #     exp_join = '''
    # public.customer INNER JOIN public.scoring_model
    # ON public.customer.id = public.scoring_model.customer_id
    # '''
    #     assert final_join == exp_join

    # def drop_relationship(schema, table, relationship):
    #     return {
    #         "type": "drop_relationship",
    #         "args": {
    #             "table": dict(schema=schema, name=table),
    #             "relationship": relationship
    #         }
    #     }

    # def untrack_table(schema, table):
    #     return {
    #         "type": "untrack_table",
    #         "args": {
    #             "table": {
    #                 "schema": schema,
    #                 "name": table
    #             },
    #             "cascade": True
    #         }
    #     }
    #

    # def track_table(schema, table):
    #     return {
    #         "type": "track_table",
    #         "args": {
    #             "schema": schema,
    #             "name": table
    #         }
    #     }

    # def create_array_relationship(relationship_name, schema, table, foreign_schema, foreign_table, foreign_column):
    #     # return {"type": "create_array_relationship", "args": {"name": "scoringModelsBycustomerId",
    #     #                                                       "table": {"name": "customer",
    #     #                                                                 "schema": "demo"},
    #     #                                                       "using": {
    #     #                                                           "foreign_key_constraint_on": {
    #     #                                                               "table": {
    #     #                                                                   "name": "scoring_model",
    #     #                                                                   "schema": "demo"},
    #     #                                                               "column": "customer_id"}}}}
    #
    #     return {"type": "create_array_relationship", "args": {"name": relationship_name,
    #                                                           "table": {"name": table,
    #                                                                     "schema": schema},
    #                                                           "using": {
    #                                                               "foreign_key_constraint_on": {
    #                                                                   "table": {
    #                                                                       "name": foreign_table,
    #                                                                       "schema": foreign_schema},
    #                                                                   "column": foreign_column}}}}
    #

    # create_array_relationship("scoringModelsBycustomerId", "demo", 'customer', 'demo', 'scoring_model', 'customer_id')

    # def create_object_relationship(relationship_name, schema, table, foreign_column):
    #     return {"type": "create_object_relationship", "args": {"name": relationship_name,
    #                                                            "table": {"name": table,
    #                                                                      "schema": schema}, "using": {
    #             "foreign_key_constraint_on": foreign_column}}}

    # create_object_relationship("customerBycustomerId", 'demo', 'scoring_model', 'customer_id')

    # def drop_table(schema, table):
    #     return {"type": "run_sql", "args": {"sql": f"DROP TABLE {schema}.{table}"}}

    # def get_object_relationship(constraint):
    #     fk = list(constraint['column_mapping'].keys())[0]
    #     label = constraint['ref_table'] + 'By' + fk
    #     return dict(label=label, schema=constraint['table_schema'], table_name=constraint['table_name'], foreign_key=fk)

    # def get_array_relationship(constraint):
    #     c = constraint
    #     table_name = c['table_name']
    #     fk = list(c['column_mapping'].keys())[0]
    #     label = table_name + 's' + 'By' + fk
    #     return dict(label=label, ref_schema=c['ref_table_table_schema'], ref_table=c['ref_table'], schema=c['table_schema'],
    #                 table_name=table_name, foreign_key=fk)
    #

    # def get_foreign_key_constraints(mappings):
    #     fkc_dicts = flatten([m['foreign_key_constraints'] for m in mappings if
    #                          'foreign_key_constraints' in m and m['foreign_key_constraints']])
    #     return fkc_dicts
    #
    #
    # def get_table_info_for_relationships(schema):
    #     return {"type": "select", "args": {"table": {"name": "hdb_table", "schema": "hdb_catalog"}, "columns": ["*.*",
    #                                                                                                             {
    #                                                                                                                 "name": "columns",
    #                                                                                                                 "columns": [
    #                                                                                                                     "*.*"],
    #                                                                                                                 "order_by": [
    #                                                                                                                     {
    #                                                                                                                         "column": "column_name",
    #                                                                                                                         "type": "asc",
    #                                                                                                                         "nulls": "last"}]}],
    #                                        "where": {"table_schema": schema},
    #                                        "order_by": [{"column": "table_name", "type": "asc", "nulls": "last"}]}}
    #
    # url = 'http://localhost:8082'
    url = 'https://146.148.70.166'
    import os

    main_path = '/Users/rabraham/Documents/dev/python/fifteenrock/main1.py'
    # print(os.path.basename(main_path))
    # print(os.path.splitext(os.path.basename(main_path))[0])

    from pathlib import Path

    # print(Path(main_path).stem)
    print(deploy({'user_name': 'rajiv'}, main_path, project='pd', function='f1', url=url))
    pass



# -*- coding: utf-8 -*-

"""Console script for fifteenrock."""
import sys
import click
from fifteenrock.cli.compute import commands as compute_commands


@click.group('compute')
def main():
    pass


main.add_command(compute_commands.compute)

# @click.command()
# def main(args=None):
#     """Console script for fifteenrock."""
#     click.echo("Replace this message by putting your code into "
#                "fifteenrock.cli.main")
#     click.echo("See click documentation at http://click.pocoo.org/")
#     return 0
#

if __name__ == "__main__":
    sys.exit(main())  # pragma: no cover



from typing import *
import click
from fifteenrock import core
from fifteenrock.lib import util
import os


@click.command()
@click.argument('command', required=True)
@click.option('--project', required=True, help='project name')
@click.option('--function', required=True, help='function name')
@click.option('--url', required=False, help='url to 15Rock cluster')
@click.option('--main_file', type=click.Path(exists=True), required=False,
              help='path to directory containing the main function')
@click.option('--dependencies', multiple=True, required=False,
              help='Files or folders to be placed in the same package at root of package')
@click.option('--requirements_file', type=click.Path(exists=True), required=False, help='Path to requirements.txt')
@click.option('--runtime', required=False, help='e.g. python')
@click.option('--runtime_version', required=False, help='e.g. 3.6')
@click.option('--project_folder', required=False, type=click.Path(exists=True),
              help='Path to the project root. This will also be the package root if specified')
@click.option('--credentials', multiple=True, required=False,
              help='key value pair of the format key=value. This will override --credentials_file')
@click.option('--credentials_file', type=click.Path(exists=True), required=False,
              help="Path to the credentials file. Either this or --credentials should be provided.")
@click.option('--config_file', type=click.Path(exists=True), required=False,
              help="JSON Path to configuration. The values will be overridden by values passed at the terminal. Note, that there won't be any merging of dicts.")
def compute(command, project: str, function: str, url: str = None, main_file: str = None,
            dependencies: Tuple[str] = None, requirements_file: str = None, runtime: str = None, runtime_version: str
            = None, project_folder: str = None, credentials: Tuple[str] = None, credentials_file: str = None,
            config_file: str = None):
    """Commands for the compute service"""

    # TODO: split credentials by key=value pair
    # TODO: Have a config file too where you can pass all the values?
    dep_list = list(dependencies)
    credentials_list = list(credentials)
    credentials_dict = util.text_to_dict(credentials_list)

    if command == 'deploy':
        core.deploy(project, function, url=url, main_file=main_file, dependencies=dep_list,
                    requirements_file=requirements_file,
                    runtime=runtime, runtime_version=runtime_version, project_folder=project_folder,
                    credentials=credentials_dict, credentials_file=credentials_file, config_file=config_file)



from typing import *
import requests
import json
import os
import zipfile
from pathlib import Path
import functools


def hello(name: str = 'Raja') -> str:
    return f"Hello, {name}"


def print_dir_values(obj):
    for d in dir(obj):

        try:
            func = getattr(obj, d)
            if callable(func):
                print(func())
            else:
                print(func)
            print('^' + str(d))
        #         print(d + ":" + func())
        except:
            pass


def post_json(a_url: str, raw_data: Dict):
    headers = {"Content-type": "application/json"}

    mandatory = dict(headers=headers, data=json.dumps(raw_data))

    inp = mandatory
    try:
        result = requests.post(a_url, **inp)
        result.raise_for_status()
        return result.json()

    except requests.exceptions.RequestException as e:
        print(e)
        if result:
            print(f'Error:{result.json()}')
        raise e


# def post(a_url: str, raw_data: Dict, headers: Dict = None, files: Dict = None):
#     mandatory = dict(headers=headers, data=json.dumps(raw_data))
#
#     inp = {**mandatory, **optional(files=files)}
#     try:
#         result = requests.post(a_url, **inp)
#         result.raise_for_status()
#         return result.json()
#
#     except requests.exceptions.RequestException as e:
#         print(e)
#         if result:
#             print(f'Error:{result.json()}')
#         raise e

def get_json(url: str, raw_data: Dict = None, headers=None):
    headers = headers or {"Content-type": "application/json"}
    response = requests.get(url, json=raw_data, headers=headers)
    return response
    # if response:
    #     return response.json()
    # else:
    #     error_text = response.text()
    #     if is_json(error_text):
    #         return response.json()
    #     else:
    #         return dict(message=error_text, type='error', code=response.status_code)


def is_json(text: str) -> bool:
    try:
        json.loads(text)
        return True
    except ValueError as e:
        return False


# def post(url: str, raw_data: Dict = None, files: Dict = None, headers: Dict = None):
#     import os
#     result = None
#     payload = raw_data
#     inp = dict()
#     if raw_data:
#         inp = {**inp, **dict(json=(None, json.dumps(payload), 'application/json'))}
#     if files:
#         file_dict = dict()
#         for k, v in files.items():
#             file_dict[k] = (os.path.basename(v), open(v, 'rb'), 'application/octet-stream')
#         # files = dict(file=(os.path.basename(file), open(file, 'rb'), 'application/octet-stream'))
#         inp = {**inp, **file_dict}
#         pass
#
#     try:
#         if headers:
#             result = requests.post(url, headers=headers, files=inp, verify=False)
#         else:
#             result = requests.post(url, files=inp, verify=False)
#         result.raise_for_status()
#         return result.json()
#
#     except requests.exceptions.RequestException as e:
#         print(e)
#         if result:
#             print(f'Error:{result.json()}')
#         raise e
#
#     pass

def post(url: str, raw_data: Dict = None, files: Dict = None, headers: Dict = None):
    import os
    result = None
    payload = raw_data
    inp = dict()
    if raw_data:
        inp = {**inp, **dict(json=(None, json.dumps(payload), 'application/json'))}
    if files:
        file_dict = dict()
        for k, v in files.items():
            file_dict[k] = (os.path.basename(v), open(v, 'rb'), 'application/octet-stream')
        # files = dict(file=(os.path.basename(file), open(file, 'rb'), 'application/octet-stream'))
        inp = {**inp, **file_dict}
        pass

    try:
        if headers:
            result = requests.post(url, headers=headers, files=inp)
        else:
            result = requests.post(url, files=inp)
        result.raise_for_status()
        return result.json()

    except requests.exceptions.RequestException as e:
        print(e)
        if result:
            print(f'Error:{result.json()}')
        raise e

    pass


def post_file(a_url: str, files: Dict):
    headers = {'Content-type': 'multipart/form-data'}

    try:
        result = requests.post(a_url, headers=headers, files=files)
        result.raise_for_status()
        return result

    except requests.exceptions.RequestException as e:
        print(e)
        if result:
            print(f'Error:{result.json()}')
        raise e


def optional(**kwargs):
    return {k: v for k, v in kwargs.items() if v}


def module_path(file_or_dir_module_or_python_object_or_file_path):
    try:
        if os.path.exists(file_or_dir_module_or_python_object_or_file_path):
            return file_or_dir_module_or_python_object_or_file_path
    except:
        if '__path__' in file_or_dir_module_or_python_object_or_file_path.__dict__:
            return file_or_dir_module_or_python_object_or_file_path.__path__[0]
        elif '__file__' in file_or_dir_module_or_python_object_or_file_path.__dict__:
            return file_or_dir_module_or_python_object_or_file_path.__file__
        elif callable(file_or_dir_module_or_python_object_or_file_path):
            if type(file_or_dir_module_or_python_object_or_file_path) == functools.partial:
                return file_or_dir_module_or_python_object_or_file_path.func.__globals__['__file__']
            else:
                return file_or_dir_module_or_python_object_or_file_path.__globals__['__file__']
            # return file_or_dir_module_or_python_object_or_file_path.__globals__['__file__']
        else:
            raise ValueError('Is {} a module at all??'.format(str(file_or_dir_module_or_python_object_or_file_path)))


def archive(output_path: Path, dir_or_files=List[str]):
    make_zipfile(output_path, dir_or_files)


def make_zipfile(output_filename, source_dirs_or_files):
    with zipfile.ZipFile(output_filename, "w", zipfile.ZIP_DEFLATED) as zip:
        for source_dir_or_file in source_dirs_or_files:
            relroot = os.path.abspath(os.path.join(source_dir_or_file, os.pardir))
            if os.path.isdir(source_dir_or_file):
                for root, dirs, files in os.walk(source_dir_or_file):
                    for file in files:
                        filename = os.path.join(root, file)
                        if os.path.isfile(filename):  # regular files only
                            arcname = os.path.join(os.path.relpath(root, relroot), file)
                            if '__pycache__' in arcname or '.pyc' in arcname:
                                continue
                            zip.write(filename, arcname)
            else:
                filename = source_dir_or_file.split('/')[-1]
                zip.write(source_dir_or_file, filename)


def file_name(path: str) -> str:
    return os.path.basename(path)


def make_requirements_file(working_dir: Path) -> Path:
    from sys import stderr
    from subprocess import run, PIPE
    cmd = ['pip', 'freeze']
    # out = run(cmd, env=env, stdout=PIPE, stderr=PIPE)
    out = run(cmd, stdout=PIPE, stderr=PIPE)
    result = out.stdout.decode('utf-8')

    if out.returncode != 0:
        print(out.stdout.decode('utf-8'))
        print(out.stderr.decode('utf-8'), file=stderr)
        raise ValueError('Cannot determine requirements')

    req_path = working_dir / 'requirements.txt'
    req_path.write_text(result)
    return req_path


def make_requirements_file_notebook(working_dir: Path) -> Path:
    from sys import stderr
    from subprocess import run, PIPE
    cmd = ['pip', 'freeze']
    # out = run(cmd, env=env, stdout=PIPE, stderr=PIPE)
    out = run(cmd, stdout=PIPE, stderr=PIPE)

    if out.returncode != 0:
        print(out.stdout.decode('utf-8'))
        print(out.stderr.decode('utf-8'), file=stderr)
        raise ValueError('Cannot determine requirements')

    result = out.stdout.decode('utf-8')
    result = result.splitlines()

    blacklisted = ['Automat', 'conda', 'pycosat', 'pycurl', 'PySocks']
    b_result = []

    for r in result:
        for b in blacklisted:
            if b not in r:
                pass
            else:
                b_result.append(r)

    new_result = [r for r in result if r not in b_result]
    new_result_str = '\n'.join(new_result)

    req_path = working_dir / 'requirements.txt'
    req_path.write_text(new_result_str)
    return req_path


def determine_project_folder() -> str:
    return 'dummpy_project_folder'


def tmp_folder(folder_name: str = None, prefix='delete_me') -> Path:
    import uuid
    import tempfile
    folder_name = folder_name or (prefix + '_' + str(uuid.uuid4()))

    tmp_dir = Path(tempfile.gettempdir()) / folder_name
    tmp_dir.mkdir(parents=True, exist_ok=True)
    return tmp_dir


def remove_dir(a_dir: Path):
    import shutil
    shutil.rmtree(a_dir)
    pass


def read_json(path: Path) -> Dict:
    with path.open('r') as json_file:
        return json.load(json_file)


def write_credentials(path: Path, credentials: Dict) -> None:
    default_creds = dict(version="v1",
                         default=credentials)
    with path.open('w') as json_file:
        json.dump(default_creds, json_file, indent=4)


def text_to_dict(credentials_list: List[str]) -> Dict:
    splits = [s.split('=') for s in credentials_list]
    return {s[0]: s[1] for s in splits}


def fail_if_no_file(main_file: str):
    from pathlib import Path

    my_file = Path(main_file)
    if not my_file.is_file():
        raise ValueError(f"File {main_file} does not exist")



# -*- coding: utf-8 -*-

"""Top-level package for fifteenrock."""


__author__ = """Rajiv Abraham"""
__email__ = 'rajiv.abraham@15rock.com'
__version__ = '0.1.29'

from .core import get_database_client, deploy
from .fr_notebook import deploy_notebook
